@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2013-2017 Laurent Montel <montel@kde.org>
+   Copyright (C) 2013-2018 Laurent Montel <montel@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -17,6 +17,7 @@
    Boston, MA 02110-1301, USA.
 */
 #include "plaintexteditor.h"
+#include "kpimtextedit_debug.h"
 
 #include "texteditor/commonwidget/textmessageindicator.h"
 #include <KIO/KUriFilterSearchProviderActions>
@@ -41,6 +42,7 @@
 #include <QScrollBar>
 #include <QApplication>
 #include <QClipboard>
+#include <QShortcut>
 
 #include <sonnet/spellcheckdecorator.h>
 
@@ -238,7 +240,7 @@ void PlainTextEditor::slotSpeakText()
     } else {
         text = toPlainText();
     }
-    //qCDebug(PIMCOMMON_LOG) << " KPIMTextEdit::TextToSpeech::self()->isReady() :" << KPIMTextEdit::TextToSpeech::self()->isReady();
+    //qCDebug(KPIMTEXTEDIT_LOG) << " KPIMTextEdit::TextToSpeech::self()->isReady() :" << KPIMTextEdit::TextToSpeech::self()->isReady();
     Q_EMIT say(text);
 }
 
@@ -616,20 +618,79 @@ bool PlainTextEditor::handleShortcut(QKeyEvent *event)
         }
         return true;
     } else if (event == QKeySequence::DeleteEndOfLine) {
-        QTextCursor cursor = textCursor();
-        QTextBlock block = cursor.block();
-        if (cursor.position() == block.position() + block.length() - 2) {
-            cursor.movePosition(QTextCursor::Right, QTextCursor::KeepAnchor);
-        } else {
-            cursor.movePosition(QTextCursor::EndOfBlock, QTextCursor::KeepAnchor);
-        }
-        cursor.removeSelectedText();
-        setTextCursor(cursor);
+        deleteEndOfLine();
         return true;
     }
     return false;
 }
 
+void PlainTextEditor::deleteEndOfLine()
+{
+    QTextCursor cursor = textCursor();
+    QTextBlock block = cursor.block();
+    if (cursor.position() == block.position() + block.length() - 2) {
+        cursor.movePosition(QTextCursor::Right, QTextCursor::KeepAnchor);
+    } else {
+        cursor.movePosition(QTextCursor::EndOfBlock, QTextCursor::KeepAnchor);
+    }
+    cursor.removeSelectedText();
+    setTextCursor(cursor);
+}
+
+void PlainTextEditor::moveLineUpDown(bool moveUp)
+{
+    QTextCursor cursor = textCursor();
+    QTextCursor move = cursor;
+    move.beginEditBlock();
+
+    bool hasSelection = cursor.hasSelection();
+
+    if (hasSelection) {
+        move.setPosition(cursor.selectionStart());
+        move.movePosition(QTextCursor::StartOfBlock);
+        move.setPosition(cursor.selectionEnd(), QTextCursor::KeepAnchor);
+        move.movePosition(move.atBlockStart() ? QTextCursor::Left: QTextCursor::EndOfBlock,
+                          QTextCursor::KeepAnchor);
+    } else {
+        move.movePosition(QTextCursor::StartOfBlock);
+        move.movePosition(QTextCursor::EndOfBlock, QTextCursor::KeepAnchor);
+    }
+    QString text = move.selectedText();
+
+    move.movePosition(QTextCursor::Right, QTextCursor::KeepAnchor);
+    move.removeSelectedText();
+
+    if (moveUp) {
+        move.movePosition(QTextCursor::PreviousBlock);
+        move.insertBlock();
+        move.movePosition(QTextCursor::Left);
+    } else {
+        move.movePosition(QTextCursor::EndOfBlock);
+        if (move.atBlockStart()) { // empty block
+            move.movePosition(QTextCursor::NextBlock);
+            move.insertBlock();
+            move.movePosition(QTextCursor::Left);
+        } else {
+            move.insertBlock();
+        }
+    }
+
+    int start = move.position();
+    move.clearSelection();
+    move.insertText(text);
+    int end = move.position();
+
+    if (hasSelection) {
+        move.setPosition(end);
+        move.setPosition(start, QTextCursor::KeepAnchor);
+    } else {
+        move.setPosition(start);
+    }
+    move.endEditBlock();
+
+    setTextCursor(move);
+}
+
 void PlainTextEditor::wheelEvent(QWheelEvent *event)
 {
     if (QApplication::keyboardModifiers() & Qt::ControlModifier) {
@@ -644,11 +705,18 @@ void PlainTextEditor::wheelEvent(QWheelEvent *event)
     QPlainTextEdit::wheelEvent(event);
 }
 
-
 void PlainTextEditor::keyPressEvent(QKeyEvent *event)
 {
+    const bool isControlClicked = event->modifiers() & Qt::ControlModifier;
+    const bool isShiftClicked = event->modifiers() & Qt::ShiftModifier;
     if (handleShortcut(event)) {
         event->accept();
+    } else if (event->key() == Qt::Key_Up && isControlClicked && isShiftClicked) {
+        moveLineUpDown(true);
+        event->accept();
+    } else if (event->key() == Qt::Key_Down && isControlClicked && isShiftClicked) {
+        moveLineUpDown(false);
+        event->accept();
     } else {
         QPlainTextEdit::keyPressEvent(event);
     }
